@@ -2,13 +2,13 @@ package org.ddahl
 
 import shallot.parameter.partition.Partition
 import shallot.parameter.Permutation
-import rscala.RPersistentReference
+import rscala.PersistentReference
 
 package object shallot {
 
-  val Version = "0.3.2"
+  val Version = "0.4.0"
 
-  val Date = "2017-05-24"
+  val Date = "2017-08-22"
 
 }
 
